package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Runaway
// Camp: Civilians

class Runaway extends Civilian {
	def static order = 14;
	def static strings = Civilian.strings.attachBundle("roles/Runaway.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		return 0;
	}
	Runaway(actor = null) {
		super(actor);
	}
	def getPriority() {
		return 15;
	}
	def getRoleName() {
		return strings.getString("runaway");
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				notice(strings.getString("notice.runaway"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.runaway") + "@" + it.name;
					};
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(type != strings.getString("command.runaway")) {
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.runaway.target", target.name));
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				return true;
			case 1:
				if(JinroGame.day == 0) {
					return true;
				}
				if(target) {
					return true;
				}
				break;
		}
		return false;
	}
	// Goes to next phase.
	def update() {
		super.update();
		if(JinroGame.day == 1) {
			return;
		}
		if(JinroGame.time == 1) {
			return;
		}
		if(Wolf.bitetarget == target || target.isWolf()) {
			killed(State.followed);
		}
	}
	// When the actor is killed.
	def killed(state) {
		if(state != State.executed) {
			if(JinroGame.day >= 2 && state == State.bitten) {
				return;
			}
			noticeKilled(strings.getString("notice.killed.normal", name));
		}
		this.state = state;
		JinroGame.addHistory(this);
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	def judge(result) {
		return result == 0 && isLive();
	}
}
